#!/usr/bin/make -f
# -*- makefile -*-

GIT_DATE :=$(shell head -1 debian/changelog | grep git | sed -r -e 's:.*git([0-9\.]+).*:\1:g')
PATCH_REV :=$(shell head -1 debian/changelog | grep sergeyd | sed -r -e 's:.*sergeyd([0-9\.]+).*:\1:g')
BUILD_DIST ?=$(shell lsb_release -si)
DIST_REL_CODE ?=$(shell lsb_release -sc)
DIST_REL_VER ?=$(shell lsb_release -sr)

D_E_V :=

ifneq (,$(GIT_DATE))
D_E_V +=+git$(GIT_DATE)
endif

D_E_V +=-SergeyD/$(PATCH_REV)
DEBIAN_EXTRA_VERSION:=$(strip $(D_E_V))
export DEBIAN_EXTRA_VERSION

# For debug
$(info DEBIAN_EXTRA_VERSION is '$(DEBIAN_EXTRA_VERSION)')

ifneq (,$(filter $(DIST_REL_CODE),wheezy jessie stretch buster precise trusty xenial bionic focal))
# I.E. NOT lucid
export DEB_HOST_MULTIARCH=$(shell dpkg-architecture -qDEB_HOST_MULTIARCH)
endif

# PIE break things
export DEB_BUILD_MAINT_OPTIONS?=hardening=-pie

CFLAGS = $(shell dpkg-buildflags --get CFLAGS)
CPPFLAGS = $(shell dpkg-buildflags --get CPPFLAGS)
CFLAGS += $(CPPFLAGS)
LDFLAGS = $(shell dpkg-buildflags --get LDFLAGS)
LDFLAGS += -Wl,-O1 -Wl,--as-needed -Wl,-z,now -Wl,-z,combreloc -Wl,-z,relro -Wl,-s

ifneq ("$(wildcard /opt/openssl101r/usr/include)","")
CFLAGS += -I/opt/openssl101r/usr/include -L/opt/openssl101r/usr/lib
LDFLAGS += -L/opt/openssl101r/usr/lib
PKG_CONFIG_PATH=/opt/openssl101r/usr/lib/pkgconfig:
export PKG_CONFIG_PATH
endif

CFLAGS += -fsigned-char -fno-strict-aliasing

CFLAGS += -D_REENTRANT -DTHREAD=1 -DOPENSSL_NO_SSL2
CFLAGS += -DHAVE_SYSTEM_TZDATA=1

ifneq (,$(filter $(DIST_REL_CODE),stretch buster bionic focal))
# Force disable PIE
CFLAGS += -fno-PIE -no-pie
CPPFLAGS += -fno-PIE -no-pie
LDFLAGS += -fno-PIE -no-pie
endif

CXXFLAGS = $(CFLAGS)

export CFLAGS CPPFLAGS CXXFLAGS LDFLAGS

export MAKELEVEL=$(shell grep -c processor -m3 /proc/cpuinfo)

# Select patchset series
SRS=trusty
CNT=trusty
ifneq (,$(filter $(DIST_REL_CODE),trusty))
SRS=trusty
endif
ifneq (,$(filter $(DIST_REL_CODE),xenial))
SRS=xenial
CNT=xenial
endif
ifneq (,$(filter $(DIST_REL_CODE),jessie))
SRS=xenial
CNT=jessie
endif
ifneq (,$(filter $(DIST_REL_CODE),stretch buster bionic focal))
SRS=stretch
CNT=stretch
endif
ifneq (,$(filter $(DIST_REL_CODE),precise))
SRS=precise
endif
ifneq (,$(filter $(DIST_REL_CODE),lucid squeeze))
SRS=lucid
endif

ifneq (,$(filter $(DIST_REL_CODE),wheezy))
SRS=wheezy

QUILT_PATCHES=$(CURDIR)/debian/patches
QUILT_SERIES=$(QUILT_PATCHES)/series
export QUILT_PATCHES QUILT_SERIES

endif

LRNOSU=$(shell logrotate -d $(CURDIR)/debian/logrotate/test-su.conf 2>&1 | grep error | grep -c "unknown option")
$(info LRNOSU is '$(LRNOSU)')
