#!/usr/bin/php52 -n
<?php
/**
 * -*- coding: utf-8 -*-
 *
 * uploadsclean.py - a script to cleanup stale PHP sessions
 *
 * Based on sessionclean by Ondřej Surý
 *
 * Copyright 2013-2015 Ondřej Surý <ondrej@sury.org> (original bash script)
 * Copyright 2018-2020 Sergey Dryabzhinsky <sergey.dryabzhinsky@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

// For this php version, and cli name
$PHP="php52";
// Do not stress server, sleep a little, in microsec
$DO_SLEEP=1000;
$PIDOF_BIN="/bin/pidof";
$RENICE_BIN="/usr/bin/renice";
$IONICE_BIN="/usr/bin/ionice";

function is_exe($fpath) {
	return is_file($fpath) && is_executable($fpath);
}

function get_pid($name) {
	global $PIDOF_BIN;
	if (is_exe($PIDOF_BIN)) {
		exec($PIDOF_BIN.' '.$name.' 2>/dev/null', $output, $return_value);
		return $output;
	}
	return array();
}

if (function_exists("proc_nice")) {
	proc_nice(5);
} else {
	if (is_exe($RENICE_BIN)) {
		system($RENICE_BIN . " -n 5 -p ".getmypid()." 2>/dev/null >/dev/null");
	}
}

if (is_exe($IONICE_BIN)) {
	system($IONICE_BIN . " -c 3 -p ".getmypid()." 2>/dev/null >/dev/null");
}

# Available SAPI config directory name
$SAPIS = array(
	"apache2" => "apache2",
	"cgi" => $PHP."-cgi",
	"fpm" => $PHP."-fpm",
	"cli" => $PHP
);

if (!ini_get("register_argc_argv")) {
	error_log("Set register_argc_argv ini parameter to true!".PHP_EOL);
	exit(1);
}

if (count($argv) < 3) {
	echo "Usage: ".$argv[0]." <search-dir> <minutes-limit>".PHP_EOL;
	exit(1);
}

# Uploads can be anywhere
# And time limits are wide
# So we suggest long enough limits in cron.d/phpXX file
# Play simple, don't use argparse
$SEARCH_DIR = $argv[1];
$MINUTES_LIMIT = (int)$argv[2];

# Iterate through all web SAPIs
foreach($SAPIS as $sapi_dir => $sapi_proc) {
	if (is_file("/etc/" . $PHP .'/'. $sapi_dir . "/php.ini")) {
		$PIDS = get_pid($sapi_proc);
		foreach($PIDS as $pid) {
			$fd_path = "/proc/" . $pid . "fd";
			if (is_dir($fd_path)) {
				// Find all opened session files in SEARCH_DIR dir
				// Maybe too much files to get'em all, read one by one
				if ($handle = opendir($fd_path)) {
					while (false !== ($fn = readdir($handle))) {
						if ($fn == "." || $fn == "..") continue;
						$lnk_path = $fd_path . '/' . $fn;
						if (is_link($lnk_path)) {
							// File may be suddenly removed
							$lnk_point = @readlink($lnk_path);
							// if file deleted it will be prefixed with ' (deleted)' and not match anyway
							if ($lnk_point && fnmatch($SEARCH_DIR."/*", $lnk_point) && strpos($lnk_point, " (deleted)") === false) {
								// Touch file
								@touch($lnk_point);
							}
						}
						usleep($DO_SLEEP);
					}
					closedir($handle);
				}
			}
		}
	}
}

$old_time = time() - $MINUTES_LIMIT*60;
if ($handle = opendir($SEARCH_DIR)) {
	while (false !== ($fn = readdir($handle))) {
		if ($fn == "." || $fn == "..") continue;
		$fn_path = $SEARCH_DIR . '/' . $fn;
		// File may be suddenly removed
		$mtime = @filemtime($fn_path);
		if ($mtime && $mtime < $old_time) {
			@unlink($fn_path);
		}
		usleep($DO_SLEEP);
	}
	closedir($handle);
}
