#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# uploadsclean.py - a script to cleanup stale PHP sessions
#
# Based on sessionclean by Ondřej Surý
#
# Copyright 2013-2015 Ondřej Surý <ondrej@sury.org> (original bash script)
# Copyright 2018 Sergey Dryabzhinsky <sergey.dryabzhinsky@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import sys
import subprocess
from time import sleep, time
from fnmatch import fnmatch

# For this php version, and cli name
PHP="php52"
DO_SLEEP=0.001
PIDOF_BIN="/bin/pidof"
IONICE_BIN="/usr/bin/ionice"

def is_exe(fpath):
	return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

def get_pid(name):
	try:
		if is_exe(PIDOF_BIN):
			return subprocess.check_output([PIDOF_BIN,name]).split()
	except:
		pass
	return []

if hasattr(os, "nice"):
	os.nice(5)

try:
	import psutil
	p = psutil.Process(os.getpid())
	p.ionice(psutil.IOPRIO_CLASS_IDLE)
	IOnice=True
except:
	IOnice=False
	pass
if not IOnice:
	if is_exe(IONICE_BIN):
		subprocess.check_output([IONICE_BIN,"-c","3","-p",str(os.getpid())])

# Available SAPI config directory name
SAPIS={"apache2":"apache2", "cgi":PHP+"-cgi", "fpm":PHP+"-fpm", "cli":PHP}

# Uploads can be anywhere
# And time limits are wide
# So we suggest long enough limits in cron.d/phpXX file
# Play simple, don't use argparse
SEARCH_DIR=sys.argv[1]
MINUTES_LIMIT=int(sys.argv[2])

# Iterate through all web SAPIs
for sapi_dir, sapi_proc in SAPIS.items():
	if os.path.isfile(os.path.join("/etc", PHP, sapi_dir, "php.ini")):
		PIDS = get_pid(sapi_proc)
		for pid in PIDS:
			fd_path = os.path.join("/proc", pid, "fd")
			if os.path.isdir(fd_path):
				for root, dirs, files in os.walk(fd_path):
					for fn in files:
						lnk_path = os.path.join(root, fn)
						if os.path.islink(lnk_path):
							try:
								lnk_point = os.readlink(lnk_path)
								# if file deleted it will be prefixed with ' (deleted)' and not match anyway
								if fnmatch(lnk_point, SEARCH_DIR+"/*") and lnk_point.find(" (deleted)") == -1:
									os.utime(lnk_point, None)
							except:
								pass
						sleep(DO_SLEEP)

old_time = time() - MINUTES_LIMIT*60
for root, dirs, files in os.walk(SEARCH_DIR):
	for fn in files:
		fn_path = os.path.join(root, fn)
		try:
			# File may be suddenly removed
			mtime = os.path.getmtime(fn_path)
			if mtime < old_time:
				os.unlink(fn_path)
		except:
			pass
		sleep(DO_SLEEP)
